#version 330
#extension GL_ARB_separate_shader_objects : enable
precision highp float;
precision highp sampler2D;
precision highp sampler2DMS;
precision highp samplerCube;

layout(location = 0) smooth in vec2 _fin_fragTexCoord;

struct SFragmentInput
{
	vec2 fragTexCoord;
};

layout(location = 0) out vec4 _fout_outColor;

struct SFragmentOutput
{
	vec4 outColor;
};

 uniform samplerCube EnvironmentMapSampler;

layout( std140) uniform UInfo
{
	vec4 Top;
	vec4 Bottom;
	vec4 Origin;
	vec4 UpAxis;
	mat4 InvMat;
	vec4 EnvironmentMapColor;
	float EnvironmentMapMipLevel;
	uint EnvironmentMapVisible;
} Info;



void FragmentMain( in SFragmentInput  fInput,  out SFragmentOutput  fOutput  )
{
	vec4 topColor =  Info.Top ;
	vec4 bottomColor =  Info.Bottom ;
	mat4 invMat =  Info.InvMat ;

	vec4 wsp =  ((invMat) * (vec4((fInput.fragTexCoord *2.0f - 1.0f), 1.f, 1.f))) ;
	vec3 dir = normalize(wsp.xyz/wsp.w -  Info.Origin .xyz);
	float up = dot(dir,  Info.UpAxis .xyz);

	vec4 envMapColor =  Info.EnvironmentMapColor ;

	fOutput.outColor = int( Info.EnvironmentMapVisible ) != 0 ? vec4( textureLod(EnvironmentMapSampler, dir, Info.EnvironmentMapMipLevel ) .rgb * envMapColor.rgb, 1) : mix(bottomColor, topColor, (1.0f + up) * 0.5f);
}


void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragTexCoord = _fin_fragTexCoord;
	FragmentMain(fInput, fOutput);
	_fout_outColor = fOutput.outColor;
}
