#version 450
#extension GL_ARB_separate_shader_objects : enable

bool any(bool r) { return r; }
bool all(bool r) { return r; }

layout(location = 0) in vec2 _vin_TexCoords;

struct SVertexInput
{
	vec2 TexCoords;
	int VertexIndex;
	uint InstanceId;
};

layout(location = 0) smooth out vec4 _vtof_fragColor;

struct SVertexOutput
{
	vec4 fragColor;
	vec4 VertexPosition;
};

layout(set = 0, binding = 0) uniform USceneInfo
{
	mat4 ViewProj;
	mat4 View;
	mat4 Proj;
	mat4 InvView;
	mat4 InvViewProj;
	mat4 InvView_FrameRhYUp;
	mat4 LHZToUser;
	vec4 ViewPosition;
	vec4 ViewDirection;
	vec4 DirLightPosition;
	vec4 DirLightColor;
	vec4 AmbientLightColor;
	vec4 SideVector;
	vec4 DepthVector;
	vec2 ZBufferLimits;
	float Handedness;
} SceneInfo;
layout(set = 1, binding = 0) readonly buffer BEnabledsOffsets
{
	uint EnabledsOffsets[];
} EnabledsOffsets;
layout(set = 1, binding = 1) readonly buffer BPositionsOffsets
{
	uint PositionsOffsets[];
} PositionsOffsets;
layout(set = 1, binding = 2) readonly buffer BSize2sOffsets
{
	uint Size2sOffsets[];
} Size2sOffsets;
layout(set = 1, binding = 3) readonly buffer BAxis0sOffsets
{
	uint Axis0sOffsets[];
} Axis0sOffsets;
layout(set = 1, binding = 4) readonly buffer BColorsOffsets
{
	uint ColorsOffsets[];
} ColorsOffsets;
layout(set = 2, binding = 0) readonly buffer BGPUSimData
{
	uint GPUSimData[];
} GPUSimData;

layout(push_constant) uniform UStagePushConstant
{
	layout(offset = 0) vec4 DrawRequest[1];
	layout(offset = 16) uint IndicesOffset;
} StagePushConstant;



vec2 get_radius(uint particleID  )
{


	return  vec2( uintBitsToFloat(GPUSimData.GPUSimData [ Size2sOffsets.Size2sOffsets [ StagePushConstant .StreamOffsetsIndex ] + particleID * 2 ]) , uintBitsToFloat(GPUSimData.GPUSimData [( Size2sOffsets.Size2sOffsets [ StagePushConstant .StreamOffsetsIndex ] + particleID * 2 ) + 1]) ) ;
}


void RotateTangents(uint particleID,  inout vec3  tangent0,  inout vec3  tangent1  )
{
}


vec4 proj_position( in vec3  position  )
{

	return  (( SceneInfo.ViewProj ) * (vec4(position, 1.0f))) ;
}


vec3 ComputeTangent1( in vec3  nrm,  in vec3  tangent0  )
{
	vec3 side =  SceneInfo.SideVector .xyz;
	vec3 depth =  SceneInfo.DepthVector .xyz;

	vec3 tangent1 = cross(nrm, tangent0) *  SceneInfo.Handedness  ;
	if (dot(tangent1, tangent1) > 1.0e-8f )
		tangent1 = normalize(tangent1);
	else
	{
		const vec3 v = -side * dot(nrm, depth) + depth * (dot(nrm, side) + 0.01f);
		tangent1 = normalize( cross(nrm, v) *  SceneInfo.Handedness  );
	}
	return tangent1;
}


void VertexBillboard( in SVertexInput  vInput,  inout SVertexOutput  vOutput, uint particleID  )
{

	const uint storageId =  StagePushConstant .StreamOffsetsIndex ;
	const vec3 worldPos =  vec3( uintBitsToFloat(GPUSimData.GPUSimData [ PositionsOffsets.PositionsOffsets [ storageId ] + particleID * 3 ]) , uintBitsToFloat(GPUSimData.GPUSimData [( PositionsOffsets.PositionsOffsets [ storageId ] + particleID * 3 ) + 1]) , uintBitsToFloat(GPUSimData.GPUSimData [( PositionsOffsets.PositionsOffsets [ storageId ] + particleID * 3 ) + 2]) ) ;
	const uint drId = 0;
	const vec4 currentDr =  StagePushConstant .DrawRequest [drId];
	const uint flags =  floatBitsToUint(currentDr.x) ;
	const uint billboarderType = flags & 7U ;

	const vec2 radius = get_radius(particleID  );
	vec3 tangent0 = vec3(0, 0, 0);
	vec3 tangent1 = vec3(0, 0, 0);
	vec2 texCoords = vInput.TexCoords;
	switch (billboarderType)
	{

	case 1U :
	{
		tangent0 =  SceneInfo.InvView_FrameRhYUp [0] .xyz * radius.x;
		tangent1 =  SceneInfo.InvView_FrameRhYUp [1] .xyz * radius.y;
		RotateTangents(particleID, tangent0, tangent1  );
		break;
	}


	case 2U :
	{
		const vec3 viewPos =  SceneInfo.ViewPosition .xyz;
		const vec3 camToParticle = normalize(worldPos - viewPos);
		const vec3 upVector = cross( SceneInfo.DepthVector .xyz,  SceneInfo.SideVector .xyz) *  SceneInfo.Handedness  ;
		const vec3 upAxis = upVector + 0.01f * dot(upVector, camToParticle) *  SceneInfo.DepthVector .xyz;

		tangent0 = normalize( cross(camToParticle, upAxis) *  SceneInfo.Handedness  );
		tangent1 = cross(tangent0, camToParticle) *  SceneInfo.Handedness  * radius.y;
		tangent0 *= radius.x;
		RotateTangents(particleID, tangent0, tangent1  );
		break;
	}


	case 3U :
	{
		const vec3 viewPos =  SceneInfo.ViewPosition .xyz;
		const vec3 camToParticle = normalize(worldPos - viewPos);


		const vec3 axis_fwd =  vec3( uintBitsToFloat(GPUSimData.GPUSimData [ Axis0sOffsets.Axis0sOffsets [ storageId ] + particleID * 3 ]) , uintBitsToFloat(GPUSimData.GPUSimData [( Axis0sOffsets.Axis0sOffsets [ storageId ] + particleID * 3 ) + 1]) , uintBitsToFloat(GPUSimData.GPUSimData [( Axis0sOffsets.Axis0sOffsets [ storageId ] + particleID * 3 ) + 2]) ) ;
		tangent0 = ComputeTangent1(camToParticle, axis_fwd  ) * radius.x;
		tangent1 = axis_fwd * 0.5f;
		break;
	}


	case 4U :
	{
		const vec3 viewPos =  SceneInfo.ViewPosition .xyz;
		const vec3 camToParticle = normalize(worldPos - viewPos);


		const vec3 axis_fwd =  vec3( uintBitsToFloat(GPUSimData.GPUSimData [ Axis0sOffsets.Axis0sOffsets [ storageId ] + particleID * 3 ]) , uintBitsToFloat(GPUSimData.GPUSimData [( Axis0sOffsets.Axis0sOffsets [ storageId ] + particleID * 3 ) + 1]) , uintBitsToFloat(GPUSimData.GPUSimData [( Axis0sOffsets.Axis0sOffsets [ storageId ] + particleID * 3 ) + 2]) ) ;
		tangent0 = ComputeTangent1(camToParticle, axis_fwd  ) * radius.x;
		tangent1 = axis_fwd * 0.5f + cross(tangent0, camToParticle) *  SceneInfo.Handedness  ;
		break;
	}


	case 5U :
	{
		const vec3 viewPos =  SceneInfo.ViewPosition .xyz;
		const vec3 camToParticle = normalize(worldPos - viewPos);


		const vec3 axis_fwd =  vec3( uintBitsToFloat(GPUSimData.GPUSimData [ Axis0sOffsets.Axis0sOffsets [ storageId ] + particleID * 3 ]) , uintBitsToFloat(GPUSimData.GPUSimData [( Axis0sOffsets.Axis0sOffsets [ storageId ] + particleID * 3 ) + 1]) , uintBitsToFloat(GPUSimData.GPUSimData [( Axis0sOffsets.Axis0sOffsets [ storageId ] + particleID * 3 ) + 2]) ) ;
		tangent0 = ComputeTangent1(camToParticle, axis_fwd  ) * radius.x;
		tangent1 = axis_fwd * 0.5f;


		tangent0 *= -1.0f;


		const float upAxisStretch = abs(texCoords.y) - 1.0f;
		tangent1 -= cross(tangent0, camToParticle) *  SceneInfo.Handedness  * upAxisStretch;
		texCoords.y = clamp(texCoords.y, -1.0f, 1.0f);
		break;
	}
	default:
		break;
	}


	tangent1 *= -1.0f;


	const vec3 bbCorner = tangent0 * texCoords.x + tangent1 * texCoords.y;
	const vec3 vertexWorldPosition = worldPos + bbCorner;

	vOutput.VertexPosition = proj_position(vertexWorldPosition  );
}

void VertexMain( in SVertexInput  vInput,  out SVertexOutput  vOutput  )
{
	const uint indicesOffset =  StagePushConstant .IndicesOffset ;

		const uint storageId =  StagePushConstant .StreamOffsetsIndex ;
		const uint particleID = vInput.InstanceId;
	float degenerate = 1.0f;



	vOutput.fragColor =  vec4( uintBitsToFloat(GPUSimData.GPUSimData [ ColorsOffsets.ColorsOffsets [ storageId ] + particleID * 4 ]) , uintBitsToFloat(GPUSimData.GPUSimData [( ColorsOffsets.ColorsOffsets [ storageId ] + particleID * 4 ) + 1]) , uintBitsToFloat(GPUSimData.GPUSimData [( ColorsOffsets.ColorsOffsets [ storageId ] + particleID * 4 ) + 2]) , uintBitsToFloat(GPUSimData.GPUSimData [( ColorsOffsets.ColorsOffsets [ storageId ] + particleID * 4 ) + 3]) ) ;
	VertexBillboard(vInput, vOutput, particleID  );

	vOutput.VertexPosition *= degenerate;
}


void main()
{
	SVertexInput vInput;
	SVertexOutput vOutput;

	vInput.TexCoords = _vin_TexCoords;
	vInput.VertexIndex = gl_VertexIndex;
	vInput.InstanceId = uint(gl_InstanceIndex);

	VertexMain(vInput, vOutput);
	_vtof_fragColor = vOutput.fragColor;
	gl_Position = vOutput.VertexPosition;
}
